//
//  BFRecordScreenImageManager.swift
//  BFRecordScreenKit
//
//  Created by 胡志强 on 2021/12/14.
//

import Foundation
import CoreMedia
import AVFoundation
import BFCommonKit

class BFRecordScreenImageManager : BFRecordScreenBaseManager{
    
    fileprivate var timerr:Timer?
    var isPlaying = false
    override func resetEnv() {
        // 重置指针
        currentAssetProgress = .zero
        super.resetEnv()

    }
    
    override func startRecord() {
        super.startRecord()
    }
    
    override func play() {
        // 找到当前要播放的音频
        timerr?.invalidate()
        
        if ((recordItem?.materialDuraion ?? .zero) - currentAssetProgress).seconds < 0.05 {
            currentAssetProgress = .zero
        }

        checkCurrentPlayStatus()
        
        let startPlayTime = Date()
        let startProgress = currentAssetProgress
        
        timerr = Timer.scheduledTimer(withTimeInterval: 0.01, repeats: true, block: {[weak self] _ in
            guard let wself = self else { return }
            // MARK: 进度计时器
            let currTime = startProgress + CMTime(seconds: Date().timeIntervalSince(startPlayTime), preferredTimescale: 1000)
            wself.dele?.progreddL.text = String(format: "%@", max(0, min(currTime.seconds, (wself.recordItem?.materialDuraion.seconds ?? 0))).formatDurationToHMS())
            wself.currentAssetProgress = min(currTime, (wself.recordItem?.materialDuraion ?? .zero))
            wself.dele?.progressThumV.progress = wself.currentAssetProgress
            wself.dele?.updateSubtitle(time: wself.currentAssetProgress)
            
            if currTime.seconds >= (wself.recordItem?.materialDuraion.seconds ?? 0) {
                wself.dele?.pause()
                wself.isPlaying = false
                return
            }
            
            wself.checkCurrentPlayStatus()
        })
        RunLoop.current.add(timerr!, forMode: .common)
    }
    
    func checkCurrentPlayStatus(){
        if let voice = recordItem?.getUsedVoices().first(where: { mod in
            (mod.startCMTime - currentAssetProgress).seconds <= 0.05 && (mod.endCMTime - currentAssetProgress).seconds > 0.05
        }){
            let su = (recordPlayer?.currentItem?.asset as? AVURLAsset)?.url.lastPathComponent != (voice.wavFilePath as NSString?)?.lastPathComponent
            if (recordPlayer?.currentItem == nil) || su || (!su && !isPlaying){
                if su { // 我需要换源地址
                    recordPlayer?.pause()
                    recordPlayer?.replaceCurrentItem(with: AVPlayerItem(url: URL(fileURLWithPath: voice.wavFilePath)))
                }
                isPlaying = true
                let tt = max(CMTime.zero, currentAssetProgress - voice.startCMTime)
                BFLog(1, message: "tt:\(tt.seconds)")
                recordPlayer?.seek(to: tt , toleranceBefore: CMTime(seconds: 1, preferredTimescale: 10000), toleranceAfter: CMTime(seconds: 1, preferredTimescale: 10000), completionHandler: {[weak self] isfinished in
                    guard let wself = self else { return }
                    
                    wself.recordPlayer?.play()
//                    if isfinished {
//                    }
                })
            }
            
        }
    }
    
    override func pause() {
        timerr?.invalidate()
        timerr = nil
        
    }
    
    override func deleteRecord(at: CMTime) {
        
        // 注：删除录音后图片素材需要回撤指针进度,同时后面录音往前挪

        if let recordItem = recordItem {
            
            let old = recordItem.materialDuraion
            recordItem.materialDuraion = recordItem.voiceStickers.reduce(.zero, { partialResult, mod in
                partialResult + (mod.endCMTime - mod.startCMTime)
            })
            if let deletevoicemodel = dele?.events.last?.deletedVoices?.first {
                currentAssetProgress = deletevoicemodel.startCMTime
            }else{
                if CMTimeCompare(at, recordItem.materialDuraion) < 0 {
                    currentAssetProgress = at
                }else{
                    currentAssetProgress = recordItem.materialDuraion
                }
            }
            
            let currDuration = old - recordItem.materialDuraion
            // 更新进度
            dele?.resetCurrentProgress()
            
            for (index, item) in recordItem.voiceStickers.enumerated() {
                if index >= dele?.isStopAtRecordRange ?? 0 {
                    // 注：开始时间减去duration or 等一前一段录音的结束时间
                    item.startCMTime = item.startCMTime - currDuration
                    item.endCMTime = item.endCMTime - currDuration
                    
                    let titlsList = recordItem.titleStickers.filter { tm in
                        tm.recordId == item.recordId
                    }
                    
                    for titleM in titlsList {
                        titleM.timelineIn = titleM.timelineIn - currDuration
                        titleM.timelineOut = titleM.timelineOut - currDuration
                    }
                    //add by ak 修改变音的数据的开始结束时间
                    let vcList = recordItem.voiceChangeStickers.filter { vc in
                        vc.recordId == item.recordId
                    }
                    for voice in vcList {
                        voice.startCMTime = voice.startCMTime - currDuration
                        voice.endCMTime = voice.endCMTime - currDuration
                    }
                }
            }
            /// 重绘录音进度视图
            dele?.resetAllIndirectionView()
        }
    }
    
    override func recordFinished(isSuccess: Bool) {
        if isSuccess{
            recordItem?.materialDuraion = recordItem?.voiceStickers.last?.endCMTime ?? .zero
            
        }
    }
    
    override func changeRecordMaterail() {}
    
    override func clean() {
        super.clean()
        filter.removeTarget(playView)
    }
    
    //MARK: - 播放逻辑
    
}
