//
//  BFVoiceSettingView.swift
//  BFRecordScreenKit
//
//  Created by ak on 2022/2/15.
//  功能：发音人设置面板

import BFCommonKit
import BFMediaKit
import BFUIKit
import Foundation
import UIKit

// 操作动作类型
public enum VoiceSettingActionType: Int {
    case voiceSettingActionCancel = 1 // 取消
    case voiceSettingActionDelete = 2 // 删除
    case VoiceSettingActionConfirm = 3 // 确认
    case VoiceSettingActionSelected = 4 // 选择了一个发音人 如用于试听
}

class BFVoiceSettingView: UIView {
    let itemSize = CGSize(width: adapterWidth(width: 52) + 9, height: 40)

    // 操作板背景
    let backView = UIButton()

    // 保存所有发音人
    public var voices: Dictionary = Dictionary<String, [PQVoiceModel]>.init() {
        didSet {
            voicesCollectView.reloadData()
        }
    }

    // 所有分类
    public var categorys: Array = Array<BFVoiceCategoryModel>.init() {
        didSet {
            BFLog(message: "发音人分类数： \(categorys.count)")
            curretnCategory = categorys.first
            curretnCategory?.isSelected = true
            catagaryCollectView.reloadData()
        }
    }

    // 操作回调  isSame 是否点击的是同一个人，进行状态切换使用
    public var voiceSettingCallBack: ((_ actionType: VoiceSettingActionType, _ selectVoice: PQVoiceModel?,_ isSame:Bool ) -> Void)?

    // 当前选择的分类
    var curretnCategory: BFVoiceCategoryModel?
    // 当前选择的发音人
    public var selectVoice: PQVoiceModel?
    
    

    lazy var line1:UIView = {
        let line1 = UIView.init(frame: CGRect(x: 0, y: catagaryCollectView.frame.minY - 1, width: cScreenWidth, height: 1))
        line1.backgroundColor = UIColor.hexColor(hexadecimal: "#272727")
        return line1
        
    }()
    lazy var line2:UIView = {
        let line2 = UIView.init(frame: CGRect(x: 0, y: catagaryCollectView.frame.maxY + 1, width: cScreenWidth, height: 1))
        line2.backgroundColor = UIColor.hexColor(hexadecimal: "#272727")
        return line2
        
    }()
    // 分类列表
    lazy var catagaryCollectView: UICollectionView = {
        let layout = UICollectionViewFlowLayout()
        layout.scrollDirection = .horizontal
        layout.itemSize = itemSize
        layout.minimumLineSpacing = 30
        layout.sectionInset = UIEdgeInsets.zero
        layout.minimumInteritemSpacing = 0
        let collectView = UICollectionView(frame: CGRect(x: 60, y: 0, width: cScreenWidth - 60, height: 40), collectionViewLayout: layout)
        collectView.isPagingEnabled = false
        collectView.showsHorizontalScrollIndicator = false
        collectView.showsVerticalScrollIndicator = false
        collectView.register(BFVoiceCategoryCell.self, forCellWithReuseIdentifier: "BFVoiceCategoryCell")
        collectView.backgroundColor = UIColor.clear

        collectView.delegate = self
        collectView.dataSource = self
        if #available(iOS 11.0, *) {
            collectView.contentInsetAdjustmentBehavior = .never
        } else {
            // Fallback on earlier versions
        }
        collectView.backgroundColor = .clear

        return collectView
    }()

    // 删除 btn
    lazy var deleteBtn: UIButton = {
        let deleteBtn = UIButton()
        deleteBtn.backgroundColor = .clear
        deleteBtn.setTitleColor(.white, for: .normal)
        deleteBtn.titleLabel?.font = UIFont.systemFont(ofSize: 16)
        deleteBtn.addTarget(self, action: #selector(deleteBtnAction), for: .touchUpInside)

        deleteBtn.setBackgroundImage(imageInRecordScreenKit(by: "voiceDeleteN"), for: .normal)
        deleteBtn.setBackgroundImage(imageInRecordScreenKit(by: "voiceDeleteS"), for: .selected)
        deleteBtn.adjustsImageWhenHighlighted = false
        return deleteBtn
    }()

    // 所有发音人列表
    lazy var voicesCollectView: UICollectionView = {
        let layout = UICollectionViewFlowLayout()
        layout.scrollDirection = .vertical
//        layout.itemSize = CGSize(width: adapterWidth(width: 60), height: 78)
        // 每行最小边距
        layout.minimumLineSpacing = 20
        // 每列最小边距
        layout.minimumInteritemSpacing = 24
        layout.sectionInset = UIEdgeInsets.zero

        let collectView = UICollectionView(frame: CGRect(x: 32, y: 118, width: cScreenWidth - 32 * 2, height: 242), collectionViewLayout: layout)
        collectView.isPagingEnabled = false
        collectView.showsHorizontalScrollIndicator = false
        collectView.showsVerticalScrollIndicator = false
        collectView.register(PQVoiceCell.self, forCellWithReuseIdentifier: "PQVoiceCell")
        collectView.backgroundColor = UIColor.clear

        collectView.delegate = self
        collectView.dataSource = self
        if #available(iOS 11.0, *) {
            collectView.contentInsetAdjustmentBehavior = .never
        } else {
            // Fallback on earlier versions
        }

        return collectView
    }()

    required init?(coder _: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }

    override func layoutSubviews() {
        super.layoutSubviews()
        backView.addCorner(roundingCorners: [.topLeft, .topRight], corner: 10)
    }

    override init(frame: CGRect) {
        super.init(frame: frame)

        backgroundColor = UIColor(red: 0, green: 0, blue: 0, alpha: 0.5)
        backView.backgroundColor = UIColor.hexColor(hexadecimal: "#121212")

        addSubview(backView)
        backView.snp.makeConstraints { make in
            make.right.equalTo(self.snp.right)
            make.bottom.equalTo(self.snp.bottom)
            make.width.equalTo(cScreenWidth)
            make.height.equalTo(340)
        }
        backView.setNeedsUpdateConstraints()

        // 取消
        let cancelBtn = UIButton()
        cancelBtn.backgroundColor = .clear
        cancelBtn.setTitle("取消", for: .normal)
        cancelBtn.setTitleColor(.white, for: .normal)
        cancelBtn.titleLabel?.font = UIFont.boldSystemFont(ofSize: 17)
        cancelBtn.addTarget(self, action: #selector(cancelAction), for: .touchUpInside)
        backView.addSubview(cancelBtn)

        cancelBtn.snp.makeConstraints { make in
            make.left.equalToSuperview().offset(18)
            make.width.equalTo(35)
            make.height.equalTo(24)
            make.top.equalToSuperview().offset(18)
        }

        // 确认
        let okBtn = UIButton()
        okBtn.backgroundColor = .clear
        okBtn.setTitle("确认", for: .normal)
        okBtn.setTitleColor(UIColor.hexColor(hexadecimal: "#389AFF"), for: .normal)
        okBtn.titleLabel?.font = UIFont.systemFont(ofSize: 17)
        okBtn.addTarget(self, action: #selector(okBtnAction), for: .touchUpInside)
        backView.addSubview(okBtn)
        okBtn.snp.makeConstraints { make in
            make.right.equalToSuperview().offset(-18)
            make.width.equalTo(35)
            make.height.equalTo(24)
            make.top.equalToSuperview().offset(18)
        }

        // 删除发音设置
        let catagaryBackView = UIView()
        catagaryBackView.backgroundColor = UIColor.hexColor(hexadecimal: "#0B0B0B")
        backView.addSubview(catagaryBackView)
        catagaryBackView.snp.makeConstraints { make in
            make.right.equalToSuperview()
            make.width.equalTo(cScreenWidth)
            make.height.equalTo(40)
            make.top.equalToSuperview().offset(58)
        }

        catagaryBackView.addSubview(catagaryCollectView)
        catagaryBackView.addSubview(line1)
        catagaryBackView.addSubview(line2)

        catagaryBackView.addSubview(deleteBtn)
        deleteBtn.snp.makeConstraints { make in
            make.left.equalToSuperview().offset(16)
            make.width.equalTo(24)
            make.height.equalTo(24)
            make.top.equalTo(cancelBtn.bottomY).offset(8)
        }

        backView.addSubview(voicesCollectView)
    }
    
    func show(){
        isHidden = false

    }

    @objc func hidden() {
        isHidden = true
    }

    @objc func cancelAction() {
        hidden()
        if voiceSettingCallBack != nil {
            voiceSettingCallBack!(.voiceSettingActionCancel, selectVoice,false)
        }
    }
    

    @objc func okBtnAction() {
        hidden()
        if voiceSettingCallBack != nil {
            voiceSettingCallBack!(.VoiceSettingActionConfirm, selectVoice,false)
        }
    }

    @objc func deleteBtnAction() {
        // 说明已经选择过发音人
        if selectVoice != nil {
            // 删除选择的发音人
            if deleteBtn.isSelected {
                selectVoice?.voiceStatue = .isNormal
                selectVoice = nil
                voicesCollectView.reloadData()
            }
            deleteBtn.isSelected = !deleteBtn.isSelected

        } else {
            cShowHUB(superView: nil, msg: "还没有选择发音人")
        }

        if voiceSettingCallBack != nil {
            voiceSettingCallBack!(.voiceSettingActionDelete, selectVoice,false)
        }
    }
    
    
    /// 刷新当前选择发音人显示状态(业务使用方调用)
    /// - Parameter voiceStatue: 状态值
    public func flushSelectVoiceStatus(voiceStatue: voiceStatue){
        selectVoice?.voiceStatue = voiceStatue
        voicesCollectView.reloadData()
        
    }
}

extension BFVoiceSettingView: UICollectionViewDelegate, UICollectionViewDataSource, UIScrollViewDelegate, UICollectionViewDelegateFlowLayout {
    func collectionView(_ collectionView: UICollectionView,
                        layout _: UICollectionViewLayout,
                        sizeForItemAt indexPath: IndexPath) -> CGSize {
        if collectionView == catagaryCollectView {
            return CGSize(width: adapterWidth(width: 52) + 9, height: 40)
        }
        return CGSize(width: adapterWidth(width: 60), height: 78)
    }

    func collectionView(_ collectionView: UICollectionView, numberOfItemsInSection _: Int) -> Int {
        if collectionView == catagaryCollectView {
            return categorys.count
        }
        return voices[curretnCategory?.cateName ?? ""]?.count ?? 0
    }

    func collectionView(_ collectionView: UICollectionView, cellForItemAt indexPath: IndexPath) -> UICollectionViewCell {
        if collectionView == catagaryCollectView {
            let cell = BFVoiceCategoryCell.setCategory(collectionView: collectionView, indexPath: indexPath)

            cell.category = categorys[indexPath.item]
            return cell
        }

        let cell = PQVoiceCell.voiceList(collectionView: collectionView, indexPath: indexPath)
        let model = voices[curretnCategory?.cateName ?? ""]?[indexPath.item]

        cell.voice = model
        return cell
    }

    func collectionView(_ collectionView: UICollectionView, didSelectItemAt indexPath: IndexPath) {
        if collectionView == catagaryCollectView {
            BFLog(message: "===开始点击\(curretnCategory?.cateName ?? "")")
            curretnCategory?.isSelected = false
            curretnCategory = categorys[indexPath.item]
            curretnCategory?.isSelected = true
            catagaryCollectView.reloadData()

            voicesCollectView.reloadData()
        } else {
            //是否点击的是同一个人
            let isSame = selectVoice == voices[curretnCategory?.cateName ?? ""]?[indexPath.item] ? true : false
         
            selectVoice?.voiceStatue = .isNormal
            selectVoice = voices[curretnCategory?.cateName ?? ""]?[indexPath.item]
            selectVoice?.voiceStatue = .isSelected
            deleteBtn.isSelected = true
            voicesCollectView.reloadData()
            
            if voiceSettingCallBack != nil {
                voiceSettingCallBack!(.VoiceSettingActionSelected, selectVoice,isSame)
            }
            

        }
    }
}
