//
//  BFCameraProgressView.swift
//  BFRecordScreenKit
//
//  Created by 胡志强 on 2022/1/17.
//

import Foundation
import BFCommonKit
import BFUIKit
import CoreMedia
import UIKit
import BFMediaKit

class BFCameraProgressView: BFProgressBaseView {
    
    var imageViews = [BFThumImageView]()
    
    // 判断是否需要分割线的(最后一个录音或者是超出5秒的缩略图不要分割线)
    var newRecordLine = false
    
    // 上次更新缩略图长度时的时间戳
    var lastUpdateTime : CMTime = .zero
    
    lazy var progressView: BFAutolayoutScrollView = {
        let sv = BFAutolayoutScrollView()
        sv.bounces = false
//        sv.backgroundColor = .clear
//        sv.backgroundColor = UIColor.hexColor(hexadecimal: "#888888",alpha:0.3)
        sv.backgroundColor = UIColor(red: 0, green: 0, blue: 0, alpha: 0.3)
        sv.decelerationRate = .fast
        sv.showsHorizontalScrollIndicator = false
        sv.delegate = self
        return sv
    }()
    
    override init(frame: CGRect) {
        super.init(frame: frame)

//        addSubview(collectionV)
//        insertSubview(collectionV, belowSubview: line)
        
        insertSubview(progressView, belowSubview: line)
    }

    override func layoutSubviews() {
        super.layoutSubviews()

        progressView.snp.makeConstraints { make in
            make.edges.equalToSuperview()
        }
    }
    
    required init?(coder _: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    /// 添加缩略图
    /// - Parameter images:
    func addThumbImages(_ img:UIImage) {
        DispatchQueue.main.async {[weak self] in
            guard let wself = self else { return }
            
            let iv = BFThumImageView(image: img)
            iv.image = img
            iv.contentMode = .scaleAspectFill
            iv.clipsToBounds = true
            iv.isHiddenBord = true
            wself.progressView.contentView.addSubview(iv)
            
            if let ind = wself.recordItem?.thumbImgs.firstIndex(where: { image in
                image == img
            }){
                if ind == 0 {
                    iv.frame = CGRect(x: cScreenWidth / 2.0, y: 0, width: 0, height: 50)
                }else{
                    if wself.imageViews.count > ind - 1 {
                        let lastiv = wself.imageViews[ind - 1]
                        if let sticker = wself.recordItem!.videoStickers.first(where: { mod in
                            mod.thumImgs!.contains(img)
                        }){
                            let wid = sticker.timelineCMOut.seconds * 70.0 / 5.0 - (lastiv.rightX - cScreenWidth / 2.0)
                            let wid0 = floor(wid * UIScreen.main.scale) /  UIScreen.main.scale
                            iv.frame = CGRect(x: lastiv.rightX, y: lastiv.y, width: wid0, height: 50)
                            BFLog(1, message: "frame: new \(wid0)")
                            if wself.newRecordLine {
                                lastiv.isHiddenBord = false
                                wself.newRecordLine = false
                            }
                            
                        }
                    }
                }
                wself.imageViews.insert(iv, at: ind)
            }
            
            wself.progressView.setNeedsLayout()
            wself.progressView.layoutIfNeeded()
            
        }
        
    }
    
    func resetThumIV() {
        imageViews.forEach { iv in
            iv.removeFromSuperview()
        }
        imageViews.removeAll()
        
        var leftah:CGFloat = cScreenWidth / 2
        if let vstickers = recordItem?.videoStickers {
            for sticker in vstickers {
                if let thumimgs = sticker.thumImgs{
                    for (ind, img) in thumimgs.enumerated(){
                        let iv = BFThumImageView(image: img)
                        iv.image = img
                        iv.contentMode = .scaleAspectFill
                        iv.clipsToBounds = true
                        iv.isHiddenBord = true
                        progressView.contentView.addSubview(iv)
                        imageViews.append(iv)
                        
                        // 计算iv的宽和偏移量
                        let wid = sticker.timelineCMOut.seconds * 70.0 / 5.0 - (leftah - cScreenWidth / 2.0)
                        let wid0 = (ind < (thumimgs.count - 1)) ? 70 : floor(wid * UIScreen.main.scale) /  UIScreen.main.scale
//                        let x = (sticker.timelineCMIn.seconds + Double(ind) * 5.0) * 70.0 / 5.0
                        iv.frame = CGRect(x: leftah, y: 0.0, width: wid0, height: 50.0)
                        leftah = iv.rightX
                    }
                    (progressView.contentView.subviews.last as? BFThumImageView)?.isHiddenBord = false
                }
            }
        }
        
        if let lastiv = imageViews.last {
            lastiv.isHiddenBord = true
        }
        
        let p = CGPoint(x: (imageViews.last?.rightX ?? 0.0) - cScreenWidth / 2.0, y: 0)

        progressView.contentOffset = p
        progressView.contentSize = CGSize(width: p.x + cScreenWidth, height: 50)
    }
    
    //
    func updateCellWidth(sticker:PQEditVisionTrackMaterialsModel, progress:CMTime) {

        if let img = sticker.thumImgs?.last {
            if let lastiv = imageViews.first(where: { v in
                v.image == img
            }){
                let wid = progress.seconds * 70.0 / 5.0 - (lastiv.x - cScreenWidth / 2.0)
                let wid0 = min(floor(wid * UIScreen.main.scale) /  UIScreen.main.scale, 70)
                
                if wid0 <= lastiv.width {
                    return
                }
                var frame = lastiv.frame
                let offsetx = wid0 - frame.size.width
                frame.size.width = wid0
                lastiv.frame = frame
                
                imageViews.forEach { iv in
                    if iv.rightX > lastiv.rightX, offsetx > 0 {
                        let frame = iv.frame
                        iv.frame = frame.offsetBy(dx: offsetx, dy: 0)
                    }
                }
            }
        }
        
        let p = CGPoint(x: progress.seconds * 70 / 5.0, y: 0)
        progressView.contentOffset = p
        //            progressView.contentSize = CGSize(width: p.x + cScreenWidth, height: 50)
        
        if let last = recordItem?.videoStickers.last {
            progressView.contentSize = CGSize(width: last.timelineCMOut.seconds * 70 / 5.0 + cScreenWidth, height: 50)
        }
//        BFLog(1, message: "insert contentSize: \(progressView.contentSize.width),offset: \(progressView.contentOffset.x)")
        
    }
    
    // 结束本次录制
    func changeSepline(img:UIImage){
        newRecordLine = true
        if img != recordItem?.thumbImgs.last {
            imageViews.first { iv in
                iv.image == img
            }?.isHiddenBord = false
        }
    }
}

extension BFCameraProgressView: UIScrollViewDelegate {
    func scrollViewDidScroll(_ scrollView: UIScrollView) {
        if isDrag {
            dragScrollProgressHandle?(false, scorllToProgress(scrollView))
        }
    }

    func scrollViewWillBeginDragging(_ scrollView: UIScrollView) {
        isDrag = true
        dragStartHandle?()
        dragScrollProgressHandle?(true, scorllToProgress(scrollView))
    }

    func scrollViewDidEndDragging(_ scrollView: UIScrollView, willDecelerate decelerate: Bool) {
        if !decelerate {
            isDrag = false
            dragEndHandle?(scorllToProgress(scrollView))
        }
    }

    func scrollViewDidEndDecelerating(_ scrollView: UIScrollView) {
        isDrag = false
        dragEndHandle?(scorllToProgress(scrollView))
    }
    
    func scorllToProgress(_ scrollView: UIScrollView) -> Float {
        let totalW = Float(Int(scrollView.contentOffset.x * UIScreen.main.scale)) / Float(Int((scrollView.contentSize.width - cScreenWidth) * UIScreen.main.scale))
        return Float(min(1, max(0, totalW)))
    }
}
